package org.ow2.isac.plugin.udpinjector;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.net.SocketException;
import java.util.Map;
import java.util.HashMap;
import org.ow2.clif.scenario.isac.plugin.SessionObjectAction;
import org.ow2.clif.scenario.isac.plugin.DataProvider;
import org.ow2.clif.scenario.isac.exception.IsacRuntimeException;
import org.ow2.clif.scenario.isac.plugin.ControlAction;
import org.ow2.clif.scenario.isac.plugin.SampleAction;
import org.ow2.clif.scenario.isac.plugin.TestAction;
import org.ow2.clif.storage.api.ActionEvent;

/**
 * Implementation of a session object for plugin ~UDPinjector~
 */
public class SessionObject
	extends DatagramSocket
	implements SessionObjectAction, DataProvider, ControlAction, SampleAction, TestAction
{
	static final int TEST_ISCONNECTED = 4;
	static final int TEST_ISNOTCONNECTED = 5;
	static final int TEST_ISBOUND = 6;
	static final int TEST_ISNOTBOUND = 7;
	static final int TEST_ISCLOSED = 11;
	static final int TEST_ISNOTCLOSED = 12;
	static final int SAMPLE_CONNECT = 2;
	static final String SAMPLE_CONNECT_PORT = "port";
	static final String SAMPLE_CONNECT_HOST = "host";
	static final int SAMPLE_SEND = 13;
	static final String SAMPLE_SEND_COMMENT = "comment";
	static final String SAMPLE_SEND_DATA = "data";
	static final int SAMPLE_RECEIVE = 14;
	static final String SAMPLE_RECEIVE_COMMENT = "comment";
	static final String SAMPLE_RECEIVE_VARIABLE = "variable";
	static final int SAMPLE_STEAL = 15;
	static final String SAMPLE_STEAL_COMMENT = "comment";
	static final String SAMPLE_STEAL_BUFFER = "buffer";
	static final String SAMPLE_STEAL_PORT = "port";
	static final int SAMPLE_CONNECT2SENDER = 16;
	static final int CONTROL_BIND = 0;
	static final String CONTROL_BIND_PORT = "port";
	static final String CONTROL_BIND_HOST = "host";
	static final int CONTROL_CLOSE = 1;
	static final int CONTROL_DISCONNECT = 3;
	static final int CONTROL_SETRCVBUFSIZE = 8;
	static final String CONTROL_SETRCVBUFSIZE_SIZE = "size";
	static final int CONTROL_SETSNDBUFSIZE = 9;
	static final String CONTROL_SETSNDBUFSIZE_SIZE = "size";
	static final int CONTROL_SETTIMEOUT = 10;
	static final String CONTROL_SETTIMEOUT_TIMEOUT = "timeout";
	static final String SEND_TYPE = "UDP SEND";
	static final String RECEIVE_TYPE = "UDP RECEIVE";
	static final String STEAL_TYPE = "UDP STEAL";
	static final String CONNECT_TYPE = "UDP CONNECT";

	static final SessionObject[] boundPorts = new SessionObject[65536];
	private Map<String,String> buffers;
	protected SocketAddress lastSenderAddress = null; 

	/**
	 * Constructor for specimen object.
	 *
	 * @param params key-value pairs for plugin parameters
	 */
	public SessionObject(Map params) throws SocketException
	{
		super((SocketAddress)null);
	}

	/**
	 * Copy constructor (clone specimen object to get session object).
	 *
	 * @param so specimen object to clone
	 */
	private SessionObject(SessionObject so) throws SocketException
	{
		super((SocketAddress)null);
		buffers = new HashMap<String,String>();
	}


	////////////////////////////////////////
	// SessionObjectAction implementation //
	////////////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#createNewSessionObject()
	 */
	public Object createNewSessionObject() throws IsacRuntimeException
	{
		try
		{
			return new SessionObject(this);
		}
		catch (SocketException ex)
		{
			throw new IsacRuntimeException("Can't create UDPinjector session object", ex);
		}
	}

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#close()
	 */
	public void close()
	{
		if (! isClosed())
		{
			if (isBound())
			{
				boundPorts[getLocalPort()] = null;
			}
			super.close();
		}
	}

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#reset()
	 */
	public void reset()
	{
		buffers.clear();
		lastSenderAddress = null;
		super.disconnect();
		close();
	}


	/////////////////////////////////
	// DataProvider implementation //
	/////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.DataProvider#doGet()
	 */
	public String doGet(String var)
	{
		return buffers.get(var);
	}

	
	//////////////////////////////////
	// ControlAction implementation //
	//////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.ControlAction#doControl()
	 */
	public void doControl(int number, Map params)
	{
		switch (number)
		{
			case CONTROL_SETTIMEOUT:
				try
				{
					setSoTimeout(Integer.parseInt((String)params.get(CONTROL_SETTIMEOUT_TIMEOUT)));
					break;
				}
				catch (SocketException ex)
				{
					throw new IsacRuntimeException("Can't set UDPinjector socket timeout", ex);
				}
			case CONTROL_SETSNDBUFSIZE:
				try
				{
					setSendBufferSize(Integer.parseInt((String)params.get(CONTROL_SETSNDBUFSIZE_SIZE)));
					break;
				}
				catch (SocketException ex)
				{
					throw new IsacRuntimeException("Can't set UDPinjector socket send buffer size", ex);
				}
			case CONTROL_SETRCVBUFSIZE:
				try
				{
					setReceiveBufferSize(Integer.parseInt((String)params.get(CONTROL_SETRCVBUFSIZE_SIZE)));
					break;
				}
				catch (SocketException ex)
				{
					throw new IsacRuntimeException("Can't set UDPinjector socket receive buffer size", ex);
				}
			case CONTROL_DISCONNECT:
				disconnect();
				break;
			case CONTROL_CLOSE:
				close();
				break;
			case CONTROL_BIND:
				try
				{
					bind(
						new InetSocketAddress(
							(String)params.get(CONTROL_BIND_HOST),
							Integer.parseInt((String)params.get(CONTROL_BIND_PORT))));
					boundPorts[getLocalPort()] = this;
				}
				catch (SocketException ex)
				{
					throw new IsacRuntimeException("Can't bind UDPinjector socket", ex);
				}
				break;
			default:
				throw new Error("Unable to find this control in ~UDPinjector~ ISAC plugin: " + number);
		}
	}


	/////////////////////////////////
	// SampleAction implementation //
	/////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SampleAction#doSample()
	 */
	public ActionEvent doSample(int number, Map params, ActionEvent report)
	{
		switch (number)
		{
			case SAMPLE_CONNECT2SENDER:
			{
				if (lastSenderAddress == null)
				{
					throw new IsacRuntimeException("Attempt to connect to a packet sender but no packet received.");
				}
				else
				{
					report.type = CONNECT_TYPE;
					return connectTo(lastSenderAddress, report);
				}
			}
			case SAMPLE_STEAL:
			{
				int port = Integer.parseInt((String)params.get(SAMPLE_STEAL_PORT));
				SessionObject target = boundPorts[port];
				if (target == null)
				{
					throw new IsacRuntimeException("Attempt to steal from unbound UDP port " + port);
				}
				else
				{
					report.type = STEAL_TYPE;
					report.comment = (String)params.get(SAMPLE_STEAL_COMMENT);
					return receiveFrom(target, report, (String)params.get(SAMPLE_STEAL_BUFFER), buffers);
				}
			}
			case SAMPLE_RECEIVE:
			{
				report.type = RECEIVE_TYPE;
				report.comment = (String)params.get(SAMPLE_RECEIVE_COMMENT);
				return receiveFrom(this, report, (String)params.get(SAMPLE_RECEIVE_VARIABLE), buffers);
			}
			case SAMPLE_SEND:
			{
				synchronized(this)
				{
					if (! isConnected())
					{
						throw new IsacRuntimeException("UDPinjector must be connected before sending data.");
					}
					byte[] data = ((String)params.get(SAMPLE_SEND_DATA)).getBytes();
					report.type = SEND_TYPE;
					report.comment = (String)params.get(SAMPLE_SEND_COMMENT);
					report.setDate(System.currentTimeMillis());
					try
					{
						send(new DatagramPacket(data, data.length));
						report.duration = (int)(System.currentTimeMillis() - report.getDate());
						report.result = String.valueOf(data.length) + " bytes";
						report.successful = true;
					}
					catch (Exception ex)
					{
						report.successful = false;
						report.result = ex.toString();
					}
				}
				return report;
			}
			case SAMPLE_CONNECT:
			{
				String host = (String)params.get(SAMPLE_CONNECT_HOST);
				int port = Integer.parseInt((String)params.get(SAMPLE_CONNECT_PORT));
				report.type = CONNECT_TYPE;
				return connectTo(new InetSocketAddress(host, port), report);
			}
			default:
				throw new Error("Unable to find this sample in ~UDPinjector~ ISAC plugin: " + number);
		}
	}


	///////////////////////////////
	// TestAction implementation //
	///////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.TestAction#doTest()
	 */
	public boolean doTest(int number, Map params)
	{
		switch (number)
		{
			case TEST_ISNOTCLOSED:
				return ! isClosed();
			case TEST_ISCLOSED:
				return isClosed();
			case TEST_ISNOTBOUND:
				return ! isBound();
			case TEST_ISBOUND:
				return isBound();
			case TEST_ISNOTCONNECTED:
				return ! isConnected();
			case TEST_ISCONNECTED:
				return isConnected();
			default:
				throw new Error("Unable to find this test in ~UDPinjector~ ISAC plugin: " + number);
		}
	}

	/////////////////////
	// utility methods //
	/////////////////////

	private ActionEvent connectTo(SocketAddress target, ActionEvent report)
	{
		report.setDate(System.currentTimeMillis());
		try
		{
			connect(target);
			if (isConnected())
			{
				report.duration = (int)(System.currentTimeMillis() - report.getDate());
				report.successful = true;
				report.result = "connected to " + target;
			}
			else
			{
				report.successful = false;
				report.comment = "Can't connect UDPinjector socket to " + target;
				report.result = "connection failed";
			}
		}
		catch (SocketException ex)
		{
			report.successful = false;
			report.comment = "Can't connect UDPinjector socket to " + target;
			report.result = ex.toString();
		}
		return report;
	}

	private static ActionEvent receiveFrom(
		SessionObject socket,
		ActionEvent report,
		String bufferName,
		Map<String, String> buffers)
	throws IsacRuntimeException
	{
		synchronized (socket)
		{
			if (! socket.isBound())
			{
				throw new IsacRuntimeException("UDPinjector must be bound before receiving data.");
			}
			byte[] buffer;
			try
			{
				buffer = new byte[socket.getReceiveBufferSize()];
			}
			catch (SocketException ex)
			{
				throw new IsacRuntimeException("Can't get size of socket receive buffer in UDPinjector.", ex);
			}
			DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
			report.setDate(System.currentTimeMillis());
			try
			{
				socket.receive(packet);
				report.duration = (int)(System.currentTimeMillis() - report.getDate());	
				report.successful = true;
				report.result = String.valueOf(packet.getLength()) + " bytes on port " + socket.getLocalPort();
				if (bufferName != null && !bufferName.equals(""))
				{
					buffers.put(bufferName, new String(buffer, 0, packet.getLength()));
				}
				socket.lastSenderAddress = packet.getSocketAddress();
			}
			catch (Exception ex)
			{
				report.successful = false;
				report.result = ex.toString();
				if (bufferName != null && !bufferName.equals(""))
				{
					buffers.remove(bufferName);
				}
			}
		}
		return report;
	}
}
