/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004, 2010 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.deploy;

import org.objectweb.fractal.api.Component;
import org.ow2.clif.server.api.ClifServerControl;
import org.ow2.clif.supervisor.api.ClifException;
import java.util.Map;


/**
 * Asynchronously and remotely instantiates a Blade component in a CLIF server.
 * @author Bruno Dillenseger
 */
public class BladeDeploy extends Thread
{
	protected Component clifApp;
	protected ClifServerControl server;
	protected String id;
	protected String adlFilename;
	protected Map<String, String> adlParameters;
	protected String argument;
	protected Component bladeComp = null;
	protected ClifException exception = null;
	protected boolean terminated = false;


	/**
	 * Defines a new Blade Component deployment.
	 * @param clifApp the Clif application distributed component where this blade will be added
	 * @param adlFilename ADL definition of a BladeType compliant component
	 * @param adlParameters parameters for ADL definition (if any)
	 * @param server the controller of the CLIF server where to deploy the blade
	 * @param id the blade identifier
	 */
	public BladeDeploy(Component clifApp, String adlFilename, Map<String, String> adlParameters, ClifServerControl server, String id)
	{
		super(adlFilename + " blade deployment on " + server + " with parameters " + adlParameters);
		this.clifApp = clifApp;
		this.server = server;
		this.adlParameters = adlParameters;
		this.adlFilename = adlFilename;
		this.id = id;
	}


	/**
	 * Actually performs the blade deployment.
	 */
	public synchronized void run()
	{
		try
		{
			bladeComp = server.addBlade(clifApp, adlFilename, adlParameters, id);
		}
		catch (ClifException ex)
		{
			ex.printStackTrace(System.err);
			exception = ex;
		}
		terminated = true;
		notify();
	}


	/**
	 * Gets the blade component (the call is blocked waiting for the blade to be
	 * actually deployed)
	 * @return the blade component reference
	 * @throws ClifException
	 */
	public synchronized Component get()
		throws ClifException
	{
		while (! terminated)
		{
			try
			{
				wait();
			}
			catch (InterruptedException ex)
			{
				ex.printStackTrace(System.err);
			}
		}
		if (exception == null)
		{
			return bladeComp;
		}
		throw exception;
	}
}
