/*
* CLIF is a Load Injection Framework
* Copyright (C) 2005, 2009, 2010 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.deploy;

import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.Stack;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;
import org.ow2.clif.console.lib.batch.BatchUtil;
import org.ow2.clif.datacollector.api.DataCollectorAdmin;
import org.ow2.clif.server.api.BladeControl;
import org.ow2.clif.server.api.ClifServerControl;
import org.ow2.clif.storage.api.AlarmEvent;
import org.ow2.clif.storage.api.CollectListener;
import org.ow2.clif.storage.api.StorageProxyAdmin;
import org.ow2.clif.supervisor.api.BladeState;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.supervisor.api.SupervisorInfo;
import org.ow2.clif.supervisor.api.TestControl;
import org.ow2.clif.supervisor.lib.BladeObservation;
import org.ow2.clif.util.CodeServer;
import org.ow2.clif.util.ExecutionContext;
import org.ow2.clif.util.ItfName;
import org.ow2.clif.util.StringHelper;


/**
 * This class is responsible for creating the initial Clif application. Then, it provides test
 * plan deployment.
 * @author Bruno Dillenseger
 * @author Joan Chaumont
 * @author Florian Francheteau
 */
public class ClifAppFacade extends Observable implements Observer
{
	private Object deployLock = new Object();
	private volatile boolean deployed = false;
	protected String clifAppDefinition;
	protected String clifAppName;
	protected String currentTestId;
	protected Component clifApp;
	protected Component storage;
	protected Component supervisor;
	/** clif application components indexed by their name (storage, supervisor, analyzer) */
	protected Map<String,Component> components = new HashMap<String,Component>();
	protected BindingController storageBc;
	protected BindingController supervisorBc;
	protected LifeCycleController clifAppLfc;
	protected ContentController clifAppCc;
	/** contains bound clif server components, indexed by their names */
	private TestControl testControl;
	private SupervisorInfo supInf;

	/**
	 * Create new ClifAppFacade with all components of a ClifApplication :
	 * supervisor, storage...
	 * @param testName the name in the Registry to be associated with the
	 * resulting Clif Application.
	 * @param appDefinition fully-qualified Fractal ADL definition file of the
	 * CLIF application to instantiate.
	 * @throws Error
	 */
	public ClifAppFacade(String testName, String appDefinition)
	{
		clifAppName = testName;
		clifAppDefinition = appDefinition;
		try {
			clifApp = (Component)FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND
					).newComponent(clifAppDefinition, null);		
			setClifAppComponents();
			clifAppLfc.startFc();
		} catch (Exception e) {
			throw new Error("Error creating Clif application " + appDefinition + " for " + testName, e);
		}
	}

	/**
	 * Constructor for ClifAppFacade. Get all sub-components
	 * from the clifApp
	 * @param clifApp the existing Clif application component
	 */
	public ClifAppFacade (Component clifApp, String name)
	{
		this.clifApp = clifApp;
		this.clifAppName = name;
		try {
			setClifAppComponents();
		} catch (Exception e) {
			throw new Error("Error creating Clif application " + name, e);
		}
	}

	/**
	 * Set clifApp sub-components
	 * @throws ADLException
	 * @throws NoSuchInterfaceException
	 * @throws IllegalLifeCycleException
	 */
	private void setClifAppComponents()
		throws ADLException, NoSuchInterfaceException, IllegalLifeCycleException {
		Component[] subComp = Fractal.getContentController(clifApp).getFcSubComponents();
		for (int i=0 ; i<subComp.length ; ++i)
		{
			components.put(Fractal.getNameController(subComp[i]).getFcName(), subComp[i]);
			if (Fractal.getNameController(subComp[i]).getFcName().equals("storage"))
			{
				storage = subComp[i];
			}
			else if (Fractal.getNameController(subComp[i]).getFcName().equals("supervisor"))
			{
				supervisor = subComp[i];
			}
		}
		storageBc = Fractal.getBindingController(storage);
		supervisorBc = Fractal.getBindingController(supervisor);
		clifAppLfc = Fractal.getLifeCycleController(clifApp);
		clifAppCc = Fractal.getContentController(clifApp);
		testControl = (TestControl)supervisor.getFcInterface(TestControl.TEST_CONTROL);
		supInf = (SupervisorInfo)ClifAppFacade.this.supervisor.getFcInterface(SupervisorInfo.SUPERVISOR_INFO);
	}

	/**
	 * Get a component of the clifApplication by his Fractal name
	 * @param name the name of the component to find
	 * @return the reference of a component contained by the Clif Application whose name equals the
	 * String passed as parameter, or null if there is no such component.
	 */
	public Component getComponentByName(String name)
	{
		return components.get(name);
	}

	/**
	 * Get clifApp component
	 * @return Returns the clifApp component.
	 */
	public Component getClifApp() {
		return clifApp;
	}

	/**
	 * Asynchronously deploys blades among CLIF servers according to the given definitions.
	 * First, a new CLIF code server is launched.
	 * Then, current CLIF application is stopped, reconfigured and started again.
	 * In case the CLIF application is doing wrong (typically one of its distributed parts
	 * is no longer available), a new one is instantiated. So, be careful to always call
	 * {@link #getClifApp()} from one deployment to another whenever you need to get the
	 * reference to the CLIF application component.
	 * Old blades that may have been previously deployed by this supervisor are removed,
	 * unless a new CLIF application is instantiated or previous blades became unreachable.
	 * DeployObservation objects will be send to Observers to inform about deployment
	 * success or failure. BladeObservation objects are also sent to notify blades state changes.
	 * @param definitions a Map containing test plan definitions, indexed by blades identifiers
	 * @param registry the CLIF registry to be used for getting CLIF servers references
	 * @see org.ow2.clif.deploy.DeployDefinition
	 * @see org.ow2.clif.deploy.DeployObservation
	 * @see org.ow2.clif.supervisor.lib.BladeObservation
	 * @see #syncDeploy(Map, ClifRegistry)
	 * @throws ClifException when the CLIF code server could not be started
	 */
	public void deploy(final Map definitions, final ClifRegistry registry)
		throws ClifException
	{
	    try
	    {
	        CodeServer.launch(null, Integer.parseInt(System.getProperty("clif.codeserver.port", "1357")),
	        	ExecutionContext.getBaseDir() + "lib/ext/", System.getProperty("clif.codeserver.path", "."));
	    }
        catch (Exception ex)
        {
            throw new ClifException("Could not start the CLIF code server.", ex);
        }
    	new Thread(
    		new Runnable() {
        		public void run()
        		{
        			try
        			{
	        			clifAppLfc.stopFc();
        			}
        	        catch (Exception ex)
        	        {
        	        	System.err.println(
        	        		"Warning: could not stop current CLIF Application: "
        	        		+ ex.getMessage()
        	        		+ "\nA new one will be created.");
        	    		try
        	    		{
        	    			// restart a new CLIF application
        	    			currentTestId = null;
        	    			clifApp = (Component)FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND)
        	    				.newComponent(clifAppDefinition, null);
        	    			setClifAppComponents();
        	    		}
        	    		catch (Exception ex2)
        	    		{
        	    			setChanged();
            	        	notifyObservers(
            	        		new DeployObservation(
            	        			false,
            	        			new ClifException(
            	        				"Deployment failed because the CLIF Application could be neither stopped nor restarted.",
            	        				ex2)));
        	    			throw new Error("Fatal error: can't restart CLIF application " + clifAppDefinition + " for " + clifAppName, ex2);
        	    		}
        	        }
        			DeployThread deployerThr = new DeployThread(definitions, registry);
        			deployerThr.start();
        		}},
        	"Test plan deployment"
    	).start();
	}

	/**
	 * Synchronous version of deploy method (returns when deployment is complete),
	 * i.e. returns when the deployment process is complete.
	 * @param definitions a Map containing test plan definitions, indexed by blades identifiers
	 * @param registry the CLIF registry to be used for getting CLIF servers references
	 * @see #deploy(Map, ClifRegistry)
	 * @see org.ow2.clif.deploy.DeployDefinition
	 * @see org.ow2.clif.deploy.DeployObservation
	 */
	public void syncDeploy(Map definitions, ClifRegistry registry)
		throws ClifException
	{
		deployed = false;
		deploy(definitions, registry);
		synchronized(deployLock)
		{
			while (!deployed)
			{
				try
				{
					deployLock.wait();
				}
				catch (InterruptedException ex)
				{
				}
			}
		}
	}

	/**
	 * Init selected blades if global state is deployed
	 * @param testId selected blades ids
	 */
	public void init(String testId)
		throws ClifException
	{
		try
		{
			supInf.waitStationaryState(null);
		}
		catch (InterruptedException ex)
		{
			throw new ClifException("Can't initialize: interrupted while waiting for a stationary state.", ex);
		}
		if (supInf.getGlobalState(null).equals(BladeState.DEPLOYED)
			|| supInf.getGlobalState(null).equals(BladeState.COMPLETED)
			|| supInf.getGlobalState(null).equals(BladeState.STOPPED))
		{
			currentTestId = testId + "_" + StringHelper.getCurrentDate();
	        testControl.init(currentTestId);
		}
		else
		{
			throw new ClifException("Can't initialize: lifecycle error");
		}
	}

    /**
	 * Start selected blades if global state is initialized
	 * @param selBladesId selected blades ids
	 * @return the int code return for exit info
	 */
	public int start(String[] selBladesId)
	{
		try
		{
			supInf.waitStationaryState(selBladesId);
			if (supInf.getGlobalState(selBladesId).equals(BladeState.INITIALIZED))
			{
				testControl.start(selBladesId);
				return BatchUtil.SUCCESS;
			}
			else
			{
				return BatchUtil.ERR_LIFECYCLE;
			}
		}
		catch (InterruptedException ex)
		{
			ex.printStackTrace(System.err);
			return BatchUtil.ERR_EXEC;
		}
	}

	/**
	 * Stop selected blades if global state is not stopped/completed/aborted
	 * @param selBladesId selected blades ids
	 * @return the int code return for exit info
	 */
	public int stop(String[] selBladesId)
	{
		try
		{
			supInf.waitStationaryState(selBladesId);
			if (!supInf.getGlobalState(selBladesId).equals(BladeState.STOPPED)
				&& !supInf.getGlobalState(selBladesId).equals(BladeState.COMPLETED)
				&& !supInf.getGlobalState(selBladesId).equals(BladeState.ABORTED))
			{
				testControl.stop(selBladesId);
				return BatchUtil.SUCCESS;
			}
			else
			{
				return BatchUtil.ERR_LIFECYCLE;
			}
		}
		catch (InterruptedException ex)
		{
			ex.printStackTrace(System.err);
			return BatchUtil.ERR_EXEC;
		}
	}

	/**
	 * Suspend selected blades if global state is running
	 * @param selBladesId selected blades ids
	 * @return the int code return for exit info
	 */
	public int suspend(String[] selBladesId)
	{
		try
		{
			supInf.waitStationaryState(selBladesId);
			if (supInf.getGlobalState(selBladesId).equals(BladeState.RUNNING))
			{
			    testControl.suspend(selBladesId);
			    return BatchUtil.SUCCESS;
			}
			else
			{
				return BatchUtil.ERR_LIFECYCLE;
			}
		}
		catch (InterruptedException ex)
		{
			ex.printStackTrace(System.err);
			return BatchUtil.ERR_EXEC;
		}
	}

	/**
	 * Resume selected blades if global state is suspended
	 * @param selBladesId selected blades ids
	 * @return the int code return for exit info
	 */
	public int resume(String[] selBladesId)
	{
		try
		{
			supInf.waitStationaryState(selBladesId);
			if (supInf.getGlobalState(selBladesId).equals(BladeState.SUSPENDED))
			{
			    testControl.resume(selBladesId);
			    return BatchUtil.SUCCESS;
			}
			else
			{
				return BatchUtil.ERR_LIFECYCLE;
			}
		}
		catch (InterruptedException ex)
		{
			ex.printStackTrace(System.err);
			return BatchUtil.ERR_EXEC;
		}
	}

	/**
	 * Join selected blades if global state is running
	 * @param selBladesId selected blades ids
	 * @return the int code return for exit info
	 */
	public int join(String[] selBladesId)
	{
		try
		{
			supInf.waitStationaryState(selBladesId);
			if (supInf.getGlobalState(selBladesId).equals(BladeState.RUNNING))
			{
				testControl.join(selBladesId);
				return BatchUtil.SUCCESS;
			}
			else
			{
				return BatchUtil.ERR_LIFECYCLE;
			}
		}
		catch (InterruptedException ex)
		{
			ex.printStackTrace(System.err);
			return BatchUtil.ERR_EXEC;
		}
	}

	/**
	 * Collect selected blades if global state is completed or stopped
	 * @param selBladesId selected blades ids
	 * @return the int code return for exit info
	 */
	public int collect(String[] selBladesId, CollectListener listener)
	{
		try
		{
			supInf.waitStationaryState(selBladesId);
			supInf.waitEndOfRun(selBladesId);
			if (supInf.getGlobalState(selBladesId).equals(BladeState.COMPLETED)
				|| supInf.getGlobalState(selBladesId).equals(BladeState.STOPPED)
				|| supInf.getGlobalState(selBladesId).equals(BladeState.ABORTED))
			{
				testControl.collect(selBladesId, listener);
				return BatchUtil.SUCCESS;
			}
			else
			{
				return BatchUtil.ERR_LIFECYCLE;
			}
		}
		catch (InterruptedException ex)
		{
			ex.printStackTrace(System.err);
			return BatchUtil.ERR_EXEC;
		}
	}

	/**
	 * Get global state for these blades.
	 * @param selBladesId the blades ids
	 * @return BladeState the global state
	 */
	public BladeState getGlobalState(String[] selBladesId)
	{
		BladeState globalState = null;
		if (ClifAppFacade.this.clifAppLfc.getFcState().equals(LifeCycleController.STARTED))
		{
			globalState = supInf.getGlobalState(selBladesId);
		}
		else
		{
			globalState = BladeState.DEPLOYING;
		}
		return globalState;
	}

    /**
     * Get the state of a blade
     * @param id the bladeId tested
     * @return the blade state
     */
	public BladeState getState(String id) {
        String [] testId = {id};
        return getGlobalState(testId);
    }

    /**
     * Waits until this state is reached
     * @param selBladesId
     * @param state
     * @return int return code
     */
    public int waitForState(String[] selBladesId, BladeState state)
    {
    	try
    	{
	        if (supInf.waitForState(selBladesId, state))
	        {
	            return BatchUtil.SUCCESS;
	        }
	        else
	        {
	        	return BatchUtil.ERR_LIFECYCLE;
	        }
	    }
    	catch (InterruptedException ex)
    	{
    		ex.printStackTrace(System.err);
    		return BatchUtil.ERR_EXEC;
    	}
    }

    /**
	 * Retrieve the ActionStat of a specific host.
	 * @param bladeId The blade identifier to get the statistical data from
	 * @return An array containing some informations about the test for this host
	 */
	public long[] getStats(String bladeId) {
        return testControl.getStats(bladeId);
	}

	/**
	 * Retrieve the ActionStat of a specific host.
	 * @param bladeId The blade identifier to get the statistics labels from
	 * @return An array containing some informations about the test for this host
	 */
	public String[] getStatLabels(String bladeId) {
		return testControl.getStatLabels(bladeId);
	}

	public Map getCurrentParameters(String id) {
	    return testControl.getCurrentParameters(id);
    }

	/**
	 * Retrieve the currentTestId.
	 * @return A string composed by the name, the date and the hour of the test
	 */
	public String getCurrentTestId() {
	    return currentTestId;
    }

    public void changeParameter(String bladeId, String name, String text)
    	throws ClifException
    {
        testControl.changeParameter(bladeId, name, text);
    }

    public void update(Observable supervisor, Object observation) {
	    if(countObservers() == 0 && observation instanceof AlarmEvent)
	    {
	        AlarmEvent alarm = (AlarmEvent)observation;
			if (alarm.argument != null)
			{
				if (alarm.argument instanceof Throwable)
				{
				    ((Throwable)alarm.argument).printStackTrace();
				    System.exit(1);
				}
			}
	    }
		setChanged();
		notifyObservers(observation);
	}

	class DeployThread extends Thread
	{
		Map definitions;
		ClifRegistry registry;


		DeployThread(Map testPlan, ClifRegistry registry)
		{
			super("ClifAppFacade deployment thread");
			this.definitions = testPlan;
			this.registry = registry;
		}


		public void run()
		{
			DeployObservation result = null;
			Map<String,Component>  serversComp;
			// step 1: try to remove existing blades
			try
			{
				serversComp = registry.getServerComponents();
				String[] itfNames;
				itfNames = ClifAppFacade.this.storageBc.listFc();

				for (String itfName1 : itfNames) {
					if (itfName1.startsWith(StorageProxyAdmin.STORAGEPROXY_ADMIN)) {
						ClifAppFacade.this.storageBc.unbindFc(itfName1);
					}
				}
				itfNames = ClifAppFacade.this.supervisorBc.listFc();
				for (String itfName : itfNames) {
					if (itfName.startsWith(DataCollectorAdmin.DATA_COLLECTOR_ADMIN) || itfName.startsWith(BladeControl.BLADE_CONTROL)) {
						ClifAppFacade.this.supervisorBc.unbindFc(itfName);
					}
				}
				if(serversComp != null)
				{
				    Stack<BladeRemove> pendingRemoval = new Stack<BladeRemove>();
					for (Map.Entry<String, Component> entry : serversComp.entrySet())
					{
						try
						{
							BladeRemove removeThr = new BladeRemove(
								clifApp,
								(ClifServerControl) entry.getValue().getFcInterface(ClifServerControl.CLIF_SERVER_CONTROL));
							removeThr.start();
							pendingRemoval.push(removeThr);
						}
						catch (Exception exc)
						{
							System.err.println("Notice: can't remove old blades in CLIF server " + entry.getKey() + ": " + exc);
						}
					}
				    while (! pendingRemoval.empty()) {
				        pendingRemoval.pop().join();
				    }
				}
			}
			catch (Exception ex)
			{
				result = new DeployObservation(false, ex);
				serversComp = null;
			}
			if (result == null && definitions != null)
			{
				try
				{
					// step 2: deploy new blades
					Map<String,BladeDeploy> pendingBlades = new HashMap<String,BladeDeploy>();
					for (Map.Entry<String, DeployDefinition> entry :(Set<Map.Entry<String, DeployDefinition>>)definitions.entrySet())
					{
						DeployDefinition def = entry.getValue();
						String key = entry.getKey();
						try
						{
							BladeDeploy deployThr = new BladeDeploy(
								ClifAppFacade.this.clifApp,
								def.getAdlDefinition(),
								def.getContext(),
								(ClifServerControl)serversComp.get(
									def.getServerName())
									.getFcInterface(ClifServerControl.CLIF_SERVER_CONTROL),
								key);
							pendingBlades.put(key, deployThr);
							deployThr.start();
							ClifAppFacade.this.setChanged();
							ClifAppFacade.this.notifyObservers(new BladeObservation(key, BladeState.DEPLOYING));
						}
						catch (Exception exc)
						{
							throw new ClifException("Can't communicate with CLIF server " + def.getServerName(), exc);
						}
					}
					for (Map.Entry<String, BladeDeploy> entry : pendingBlades.entrySet())
					{
						try
						{
							Component blade = entry.getValue().get();
							if (blade != null) {
								Fractal.getBindingController(blade).bindFc(
									SupervisorInfo.SUPERVISOR_INFO,
									ClifAppFacade.this.supervisor.getFcInterface(SupervisorInfo.SUPERVISOR_INFO));
								ClifAppFacade.this.storageBc.bindFc(
									ItfName.gen(StorageProxyAdmin.STORAGEPROXY_ADMIN),
									blade.getFcInterface(StorageProxyAdmin.STORAGEPROXY_ADMIN));
								ClifAppFacade.this.supervisorBc.bindFc(
									ItfName.gen(DataCollectorAdmin.DATA_COLLECTOR_ADMIN),
									blade.getFcInterface(DataCollectorAdmin.DATA_COLLECTOR_ADMIN));
								ClifAppFacade.this.supervisorBc.bindFc(
									ItfName.gen(BladeControl.BLADE_CONTROL),
									blade.getFcInterface(BladeControl.BLADE_CONTROL));
							}
							else
							{
								setChanged();
								notifyObservers(new BladeObservation(entry.getKey(), BladeState.UNDEPLOYED));
								result = new DeployObservation(
									false,
									new ClifException(
										"Clif server failed creating blade " + entry.getKey() + " for unexpected reason."));
							}
						}
						catch (ClifException ex)
						{
							// must be processed in next step
						}
					}
					// step 3: starts CLIF application and set blades arguments
					ClifAppFacade.this.clifAppLfc.startFc();
					testControl.deleteObservers();
					if (result == null)
					{
						for (Map.Entry<String, BladeDeploy> entry : pendingBlades.entrySet())
						{
							String bladeId = entry.getKey();
							try
							{
								Component blade = entry.getValue().get();
								if (blade != null)
								{
									BladeControl bladeCtl = (BladeControl) blade.getFcInterface(BladeControl.BLADE_CONTROL);
									bladeCtl.setId(bladeId);
									bladeCtl.setArgument(((DeployDefinition)definitions.get(bladeId)).getArgument());
									supInf.setBladeState(bladeId, BladeState.DEPLOYED);
									ClifAppFacade.this.setChanged();
									ClifAppFacade.this.notifyObservers(new BladeObservation(bladeId, BladeState.DEPLOYED));
								}
							}
							catch (ClifException ex)
							{
								setChanged();
								supInf.setBladeState(bladeId, BladeState.UNDEPLOYED);
								notifyObservers(new BladeObservation(bladeId, BladeState.UNDEPLOYED));
								result = new DeployObservation(false, ex);
							}
						}
						ClifAppFacade.this.supInf.setDefinitions(definitions);
					}
					testControl.addObserver(ClifAppFacade.this);
				}
				catch (NoSuchInterfaceException ex)
				{
					result = new DeployObservation(
						false,
						new Error(
							"Fatal error in blade deployment: Clif server type mismatch",
							ex));
				}
				catch (Exception ex)
				{
					result = new DeployObservation(false, new ClifException("Deployment failure: " + ex.getMessage(), ex));
				}
			}
			if (ClifAppFacade.this.clifAppLfc.getFcState().equals(LifeCycleController.STOPPED))
			{
				try
				{
					ClifAppFacade.this.clifAppLfc.startFc();
				}
				catch (IllegalLifeCycleException ex)
				{
					result = new DeployObservation(
						false,
						new ClifException(
							"Deploy failed and CLIF application could not be restarted: " + ex.toString(),
							result.getException()));
				}
			}
			if (result == null)
			{
				result = new DeployObservation();
			}
			ClifAppFacade.this.setChanged();
			ClifAppFacade.this.notifyObservers(result);
			synchronized(ClifAppFacade.this.deployLock)
			{
				ClifAppFacade.this.deployed = true;
				ClifAppFacade.this.deployLock.notify();
			}
		}
	}
}
