/*
 * CLIF is a Load Injection Framework Copyright (C) 2005 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * CLIF
 * 
 * Contact: clif@ow2.org
 */
package org.ow2.clif.scenario.isac.engine;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import org.ow2.clif.datacollector.api.DataCollectorWrite;
import org.ow2.clif.scenario.isac.engine.instructions.ControlInstruction;
import org.ow2.clif.scenario.isac.engine.instructions.GotoInstruction;
import org.ow2.clif.scenario.isac.engine.instructions.Instruction;
import org.ow2.clif.scenario.isac.engine.instructions.NChoiceInstruction;
import org.ow2.clif.scenario.isac.engine.instructions.ParamsHolder;
import org.ow2.clif.scenario.isac.engine.instructions.PlugInParamPart;
import org.ow2.clif.scenario.isac.engine.instructions.SampleInstruction;
import org.ow2.clif.scenario.isac.engine.instructions.TestInstruction;
import org.ow2.clif.scenario.isac.engine.instructions.TimerInstruction;
import org.ow2.clif.scenario.isac.plugin.ControlAction;
import org.ow2.clif.scenario.isac.plugin.DataProvider;
import org.ow2.clif.scenario.isac.plugin.SampleAction;
import org.ow2.clif.scenario.isac.plugin.TestAction;
import org.ow2.clif.scenario.isac.plugin.TimerAction;
import org.ow2.clif.server.api.BladeInsertResponse;
import org.ow2.clif.storage.api.ActionEvent;
import org.ow2.clif.storage.api.AlarmEvent;
import org.ow2.clif.util.ClifClassLoader;
import org.ow2.clif.util.ThrowableHelper;

/**
 * @author Emmanuel Varoquaux
 * @author Bruno Dillenseger
 */
class ExecutionThread extends Thread {
	/* Parameters */
	private final Clock					clock;
	private final Scheduler				scheduler;
	private final DataCollectorWrite	dataCollectorWrite;
	private final BladeInsertResponse	bladeInsertResponse;

	/* Fields */
	private static volatile long		nextId;
	private static final Random			random	= new Random();
	private volatile Job				job;
	private volatile boolean			interrupted;

	/* Scheduling data */
	protected ExecutionThread			previous;
	protected ExecutionThread			next;

	/* Constructors */

	protected ExecutionThread(ThreadGroup group, Clock clock,
			Scheduler scheduler, DataCollectorWrite dataCollectorWrite,
			BladeInsertResponse bladeInsertResponse) {
		super(group, "Execution thread " + nextId++);
		this.clock = clock;
		this.scheduler = scheduler;
		this.dataCollectorWrite = dataCollectorWrite;
		this.bladeInsertResponse = bladeInsertResponse;
		setContextClassLoader(ClifClassLoader.getClassLoader());
	}

	/* Various private methods */

	private void warning(String plugInId, String method, Throwable t,
			String action) {
		StringBuffer tmp = new StringBuffer("Job ").append(job.id).append(": ")
				.append(method).append(" failed in plug-in \"")
				.append(plugInId).append("\": ").append(ThrowableHelper.toString(t));
		System.err.println(new StringBuffer("Warning: ").append(tmp));
		bladeInsertResponse.alarm(new AlarmEvent(System.currentTimeMillis(),
				AlarmEvent.WARNING, tmp.append(". ").append(action)));
	}

	private String collapse(PlugInParamPart[] parts)
	{
		StringBuilder buf = new StringBuilder();
		for (int i = 0, len = parts.length; i < len; i++)
		{
			switch (parts[i].type)
			{
			case PlugInParamPart.STRING:
				buf.append(parts[i].str);
				break;
			case PlugInParamPart.SESSION_ID:
				buf.append(job.id);
				break;
			case PlugInParamPart.CONTEXT_CALL:
				try
				{
					buf.append(((DataProvider)job.sessionObjectMap
						.get(parts[i].plugInId)).doGet(parts[i].variable));
				}
				catch (Throwable t)
				{
					warning(parts[i].plugInId, "doGet", t, "Ignore.");
				}
				break;
			case PlugInParamPart.PROPERTY:
				buf.append(System.getProperty(parts[i].variable, ""));
				break;
			}
		}
		return buf.toString();
	}

	private Map<String,String> collapse(ParamsHolder paramsHolder)
	{
		if (!paramsHolder.isSplit())
		{
			return paramsHolder.getPlainParams();
		}
		else
		{
			Map<String,String> map = new HashMap<String,String>();
			for (
				Iterator<Map.Entry<String,PlugInParamPart[]>> i = paramsHolder.getSplitParams().entrySet().iterator();
				i.hasNext();)
			{
				Map.Entry<String,PlugInParamPart[]> entry = i.next();
				map.put(entry.getKey(), collapse(entry.getValue()));
			}
			return map;
		}
	}

	private void doWork() {

		/* Here is the virtual machine proper ! */
		for (; !interrupted && !clock.isStopped();) {
			Instruction inst = job.group.getCode().get(job.ip);
			switch (inst.getType()) {
			case Instruction.SAMPLE:
				SampleInstruction sampleInst = (SampleInstruction)inst;
				try {
					ActionEvent report = new ActionEvent();
					report.sessionId = job.id;
					report = ((SampleAction)job.sessionObjectMap.get(sampleInst.plugInId)).doSample(
						sampleInst.actionNumber,
						collapse(sampleInst.paramsHolder), report);
					if (report != null)
						dataCollectorWrite.add(report);
				}
				catch (Throwable t) {
					warning(sampleInst.plugInId, "doSample", t, "Ignore.");
				}
				job.ip++;
				continue;
			case Instruction.TIMER:
				TimerInstruction timerInst = (TimerInstruction)inst;
				job.ip++;
				try {
					scheduler
							.sleep(job, ((TimerAction)job.sessionObjectMap
									.get(timerInst.plugInId)).doTimer(
									timerInst.actionNumber,
									collapse(timerInst.paramsHolder)));
				}
				catch (Throwable t) {
					warning(timerInst.plugInId, "doTimer", t, "Destroy job.");
					job.free();
				}
				return;
			case Instruction.CONTROL:
				ControlInstruction controlInst = (ControlInstruction)inst;
				try {
					((ControlAction)job.sessionObjectMap
							.get(controlInst.plugInId)).doControl(
							controlInst.actionNumber,
							collapse(controlInst.paramsHolder));
				}
				catch (Throwable t) {
					warning(controlInst.plugInId, "doControl", t, "Ignore.");
				}
				job.ip++;
				continue;
			case Instruction.TEST:
				TestInstruction testInst = (TestInstruction)inst;
				try {
					if (!((TestAction)job.sessionObjectMap
							.get(testInst.plugInId)).doTest(
							testInst.actionNumber, collapse(testInst.paramsHolder)))
						job.ip = testInst.labelFalse;
					else
						job.ip++;
				}
				catch (Throwable t) {
					warning(testInst.plugInId, "doTest", t, "Assume false.");
					job.ip = testInst.labelFalse;
				}
				continue;
			case Instruction.GOTO:
				job.ip = ((GotoInstruction)inst).label;
				continue;
			case Instruction.NCHOICE:
				NChoiceInstruction nChoiceInst = (NChoiceInstruction)inst;
				int tirage,
				i;
				synchronized (random) {
					tirage = random.nextInt(nChoiceInst.total);
				}
				for (i = 0; i < nChoiceInst.choices.size() - 1
						&& tirage >= ((NChoiceInstruction.Choice)nChoiceInst.choices
								.get(i)).proba; i++);
				job.ip = ((NChoiceInstruction.Choice)nChoiceInst.choices.get(i)).label;
				continue;
			case Instruction.EXIT:
				scheduler.free(job);
				return;
			default:
				throw new Error("Unknown instruction");
			}
		}
		scheduler.sleep(job, 0);
	}

	/* Interface */

	protected void work(Job job) {
		this.job = job;
	}

	/* run method */

	public void run() {
		while (!interrupted) {
			try {
				scheduler.getJob();
			}
			catch (InterruptedException e) {
				return;
			}
			doWork();
		}
	}

	/* interrupt method */

	public void interrupt() {
		interrupted = true;
		super.interrupt();
	}
}
