/*
 * CLIF is a Load Injection Framework Copyright (C) 2005 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * CLIF
 * 
 * Contact: clif@ow2.org
 */
package org.ow2.clif.scenario.isac.engine;

import java.util.ArrayList;
import org.objectweb.fractal.rmi.RemoteException;
import org.ow2.clif.server.api.BladeInsertResponse;
import org.ow2.clif.storage.api.AlarmEvent;

/**
 * Implementation of JobQueue using a heap.
 * 
 * @author Emmanuel Varoquaux
 */
/* All methods are synchronized. */
class JobQueue {
	/* Parameters */
	private final Clock		clock;

	/* Fields */
	private final ArrayList	heap	= new ArrayList(1000);
	private volatile long	delay;

	protected JobQueue(Clock clock, long delay) {
		this.clock = clock;
		this.delay = delay;
	}

	private void swap(int index1, int index2) {
		Object tmp;

		tmp = heap.get(index1);
		heap.set(index1, heap.get(index2));
		heap.set(index2, tmp);
	}

	private void upHeap(int index) {
		int tmp;

		if (index == 0)
			return;
		tmp = (index - 1) >> 1;
		if (((Job)heap.get(tmp)).alarm <= ((Job)heap.get(index)).alarm)
			return;
		swap(tmp, index);
		upHeap(tmp);
	}

	private void downHeap(int index) {
		int size, tmp;
		long alarm0, alarm1, alarm2;

		size = heap.size();
		tmp = (index << 1) + 1;
		if (tmp >= size)
			return;
		alarm0 = ((Job)heap.get(index)).alarm;
		alarm1 = ((Job)heap.get(tmp)).alarm;
		if (tmp + 1 < size) {
			alarm2 = ((Job)heap.get(tmp + 1)).alarm;
			if (alarm0 <= alarm1 && alarm0 <= alarm2)
				return;
			if (alarm1 > alarm2)
				tmp++;
		}
		else if (alarm0 <= alarm1)
			return;
		swap(tmp, index);
		downHeap(tmp);
	}

	protected synchronized void insert(Job job) {
		heap.add(job);
		upHeap(heap.size() - 1);
	}

	private void remove(int index) {
		int lastIndex = heap.size() - 1;

		heap.set(index, heap.get(lastIndex));
		heap.remove(lastIndex);
		downHeap(index);
	}

	protected synchronized void remove(Group group, int n) {
		Job j;

		for (int i = heap.size() - 1; i >= 0 && n > 0; i--)
			if ((j = (Job)heap.get(i)).group == group) {
				remove(i);
				j.free();
				n--;
			}
	}

	protected synchronized Job pop() {
		Job j;

		if (heap.size() == 0 || (j = (Job)heap.get(0)).alarm > clock.getDate())
			return null;
		remove(0);
		return j;
	}

	protected void setDelay(long delay) {
		this.delay = delay;
	}
	
	private void checkAux(long date, BladeInsertResponse bladeInsertResponse,
			int index) {
		Job j;

		if (delay < 0 || index >= heap.size()
				|| (j = ((Job)heap.get(index))).alarm > date - delay)
			return;
		try
		{
			bladeInsertResponse.alarm(new AlarmEvent(System.currentTimeMillis(),
				AlarmEvent.WARNING, new StringBuffer("Job ").append(j.id)
					.append(" is waiting for more than ").append(
						date - j.alarm).append(" ms.").toString()));
		}
		catch (RemoteException ex)
		{
			ex.printStackTrace(System.err);
		}
		checkAux(date, bladeInsertResponse, (index << 1) + 1);
		checkAux(date, bladeInsertResponse, (index << 1) + 2);
	}

	protected synchronized void check(BladeInsertResponse bladeInsertResponse) {
		/* We suppose that the date (in ms) won't change during checking. */
		long date = clock.getDate();

		checkAux(date, bladeInsertResponse, 0);
	}

	protected synchronized void free() {
		for (int i = 0, size = heap.size(); i < size; i++)
			((Job)heap.get(i)).free();
		heap.clear();
	}
}
