/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004, 2008 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.util;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.Socket;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import org.ow2.clif.util.bytearray.ByteArrayURLStreamHandler;

/**
 * Special classloader bound to the CodeServer utility, enabling network download
 * of classes and resources.
 * @see CodeServer
 * @author Bruno Dillenseger
 */
public class ClifClassLoader extends ClassLoader
{
	static private ClifClassLoader cl = new ClifClassLoader();


	static public ClifClassLoader getClassLoader()
	{
		return cl;
	}


	static public void clear()
	{
		cl.my_cache.clear();
		cl.loadedClasses = null;
		cl = new ClifClassLoader();
	}


	static public synchronized void fetchClassDependencies(Class<?> clazz)
	{
		if (clazz != null && !cl.loadedClasses.contains(clazz))
		{
			Class<?>[] classes = clazz.getClasses();
			for (int i = 0 ; i < classes.length ; ++i)
			{				
				fetchClassDependencies(classes[i]);
			}
			cl.loadedClasses.add(clazz);
		}
	}


	Map my_cache = new HashMap();
	Set<Class<?>> loadedClasses = new HashSet<Class<?>>();
	Socket sock = null;
	DataInputStream dins;
	DataOutputStream douts;


	private ClifClassLoader()
	{
	    super(ClifClassLoader.class.getClassLoader());
	}


	public synchronized byte[] getBytes(String name)
		throws IOException
	{
		if (sock == null)
		{
			sock = new Socket(
				System.getProperty("clif.codeserver.host"),
				Integer.parseInt(System.getProperty("clif.codeserver.port")));
			dins = new DataInputStream(sock.getInputStream());
			douts = new DataOutputStream(sock.getOutputStream());
		}
		byte[] result = null;
		// try to get the bytes from the cache (in case it has already been downloaded)
		result = (byte[])my_cache.get(name);
		if (result == null && sock != null)
		{
			douts.writeUTF(name);
			douts.flush();
			int length = dins.readInt();
			if (length >= 0)
			{
				result = new byte[length];
				int offset = 0;
				while (length > 0)
				{
					int n = dins.read(result, offset, length);
					length -= n;
					offset += n;
				}
				my_cache.put(name, result);
			}
			else if (length == CodeServer.NO_SUCH_RESOURCE)
			{
				throw new FileNotFoundException("Code server could not find " + name);
			}
			else if (length == CodeServer.RESOURCE_TOO_BIG)
			{
				throw new IOException("Resource or class " + name + " is too big");
			}
			else
			{
				throw new IOException("Code server cannot deliver " + name + " for unknown reason");
			}
		}
		return result;
	}


	protected synchronized Class findClass(String name)
		throws ClassNotFoundException
	{
		if (name.startsWith("org.objectweb.fractal"))
		{
			return getParent().loadClass(name);
		}
		Class result = null;
		String pathname = name.replace('.', '/') + ".class";
		// first, see if the class can be found using the classpath
		try
		{
			ClassLoader parent = getParent();
			if (parent == null)
			{
				parent = getSystemClassLoader();
			}
			result = parent.loadClass(name);
		}
		catch (ClassNotFoundException e)
		{
			// second, see if the class has already been defined
			result = (Class)my_cache.get(name);
			if (result == null)
			{
				byte[] code = null;
				// then, try to get the bytes and define the class
				try
				{
					code = getBytes(pathname);
				}
				catch (IOException ex)
				{
					throw new ClassNotFoundException(name, ex);
				}
				if (code != null)
				{
					result = defineClass(name, code, 0, code.length);
					my_cache.put(name, result);
				}
				else
				{
					throw new ClassNotFoundException(name);
				}
			}
		}
		return result;
	}


	public URL findResource(String name)
	{
		try
		{
			return new URL(null, "bytearray:" + name, new ByteArrayURLStreamHandler(this));
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			return null;
		}
	}


	/**
	 * @param name the resource's name
	 * @return an empty or single-value URL enumeration for the named resource
	 * @see #findResource(String)
	 */
	public Enumeration findResources(String name)
	{
		Vector result = new Vector(1);
		URL url = findResource(name);
		if (url != null)
		{
			result.add(url);
		}
		return result.elements();
	}
}
