package org.ow2.isac.plugin.synchro;

import org.ow2.clif.scenario.isac.plugin.SessionObjectAction;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;
import java.lang.Error;
import java.util.Map;
import org.ow2.clif.scenario.isac.plugin.ControlAction;
import org.ow2.clif.scenario.isac.exception.IsacRuntimeException;
import org.ow2.clif.scenario.isac.plugin.TestAction;
import org.ow2.clif.scenario.isac.util.ParameterParser;
import org.ow2.clif.scenario.isac.plugin.DataProvider;

/**
 * Implementation of a session object for plugin ~Synchro~
 */
public class SessionObject implements SessionObjectAction, ControlAction, TestAction, DataProvider {

	static final int TEST_WASNOTIFIED = 2;
	static final String TEST_WASNOTIFIED_LOCK = "lock";
	static final String TEST_WASNOTIFIED_OPTIONS = "options";
	static final int CONTROL_NOTIFY = 0;
	static final String CONTROL_NOTIFY_LOCK = "lock";
	static final int CONTROL_WAIT = 1;
	static final String CONTROL_WAIT_TIMEOUT = "timeout";
	static final String CONTROL_WAIT_LOCK = "lock";

	static protected volatile Set<String> locks = new HashSet<String>();


	////////////////////////////////
	// Synchronization primitives //
	////////////////////////////////


	/**
	 * Blocks until the given lock name belongs to the lock set.
	 * @param lockName the name of the lock to wait for
	 * @param timeout the maximum waiting time in ms, or 0 for infinite time limit
	 */
	static protected synchronized void wait(String lockName, long timeout)
		throws IsacRuntimeException
	{
		long date = System.currentTimeMillis();
		while (!locks.contains(lockName) && (timeout <= 0 || timeout + date > System.currentTimeMillis()))
		{
			try
			{
				if (timeout <= 0)
				{
					SessionObject.class.wait();
				}
				else
				{
					SessionObject.class.wait(timeout);
				}
			}
			catch(InterruptedException ex)
			{
				throw new IsacRuntimeException("Interrupted synchronization wait on lock " + lockName);
			}
		}
	}


	/**
	 * Adds the given lock name to the lock set, and notifies all threads
	 * waiting for this lock.
	 * @param lockName the name of the lock to release
	 */
	static protected synchronized void notify(String lockName)
	{
		locks.add(lockName);
		SessionObject.class.notifyAll();
	}


	/**
	 * Constructor for specimen object.
	 *
	 * @param params key-value pairs for plugin parameters
	 */
	public SessionObject(Hashtable<String, String> params)
	{
		// Specimen objects are created at initialization time only.
		// So this is a convenient way to make sure the set of locks is empty
		// before starting the ISAC scenarios, even though the set of locks
		// is generally going to be cleared more than once at initialization time
		// (which is more than necessary but actually not a problem).
		locks.clear();
	}


	////////////////////////////////////////
	// SessionObjectAction implementation //
	////////////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#createNewSessionObject()
	 */
	public Object createNewSessionObject() {
		// No need for a dedicated session object per virtual user: the state is just the class' state
		return this;
	}

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#close()
	 */
	public void close() {

	}

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#reset()
	 */
	public void reset() {

	}

	
	//////////////////////////////////
	// ControlAction implementation //
	//////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.ControlAction#doControl()
	 */
	public void doControl(int number, Map params)
	{
		switch (number)
		{
			case CONTROL_WAIT:
				wait(
					(String)params.get(CONTROL_WAIT_LOCK),
					Long.parseLong((String)params.get(CONTROL_WAIT_TIMEOUT)));
				break;
			case CONTROL_NOTIFY:
				notify((String)params.get(CONTROL_NOTIFY_LOCK));
				break;
			default:
				throw new Error(
					"Unable to find this control in ~Synchro~ ISAC plugin: "
					+ number);
		}
	}

	
	///////////////////////////////
	// TestAction implementation //
	///////////////////////////////


	/**
	 * @see org.ow2.clif.scenario.isac.plugin.TestAction#doTest()
	 */
	public boolean doTest(int number, Map params) {
		switch (number) {
			case TEST_WASNOTIFIED:
				boolean response = locks.contains((String)params.get(TEST_WASNOTIFIED_LOCK));
				if (ParameterParser.getCheckBox((String)params.get(TEST_WASNOTIFIED_OPTIONS)).equals("not"))
				{
					return ! response;
				}
				else
				{
					return response;
				}
			default:
				throw new Error(
					"Unable to find this test in ~Synchro~ ISAC plugin: "
							+ number);
		}
	}


	
	/////////////////////////////////
	// DataProvider implementation //
	/////////////////////////////////


	/**
	 * @see org.ow2.clif.scenario.isac.plugin.DataProvider#doGet()
	 */
	public synchronized String doGet(String var)
	{
		if (var.equals(""))
		{
			return locks.toString();
		}
		else
		{
			return locks.contains(var) ? var : "";
		}
	}
}
