/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Gr�gory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.egui.wizards;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.*;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;

/**
 * Provide a wizard to create a new template report.
 * @author Gr�gory CALONNIER
 * @author Jordan BRUNIER
 */
public class CLIFReportTemplateNewWizard extends Wizard implements INewWizard {	
	
	private static final String OPENING_FILE_FOR_EDITING = "Open file for editing...";
	private static final String FILE_EXTENSION = "rpttemplate";
	
	private CLIFReportSettingPage settingPage;
	private CLIFReportNewWizardPage page;
	private BirtController birtController;

	private ISelection selection;
	
	
	/**
	 * Constructor of CLIFReportTemplateNewWizard.
	 */
	public CLIFReportTemplateNewWizard( )
	{
		super();
		setNeedsProgressMonitor(true);		
		setWindowTitle("New CLIF Report Template");
		birtController=new BirtController();
	}


	///////////////////////////
	// abstract class Wizard //
	///////////////////////////

	private boolean isClifAvaible(){
		try {
			Class.forName("org.ow2.clif.analyze.lib.egui.wizards.BirtController");
		} catch (NoClassDefFoundError e) {
			return false;
		} catch (ClassNotFoundException e) {
			return false;
		}
		return true;
	}

	/**
	 * Adding pages to the wizard.
	 */
	public void addPages() {
		if(!isClifAvaible()){
			MessageDialog.openError(getShell(),
					"CLIF Console Plug-in",
					"You need to install Birt to use Clif report template");
			dispose();
			return;
		}
		page = new CLIFReportNewWizardPage(selection, CLIFReportNewWizardPage.TYPE_RPTTEMPLATE);
		addPage(page);

		settingPage = new CLIFReportSettingPage();
		addPage(settingPage);
	}

	/**
	 * This method is called when 'Finish' button is pressed in the wizard. We
	 * will create an operation and run it using wizard as execution context.
	 * @return boolean 
	 */
	public boolean performFinish() {
		final String containerName = page.getContainerName();	
		
		String fn = page.getFileName();
		final String fileName;
		if ( !fn.endsWith( "." + FILE_EXTENSION ) ) //$NON-NLS-1$
		{
			fileName = fn + "." + FILE_EXTENSION; //$NON-NLS-1$
		}
		else
		{
			fileName = fn;
		}

		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor)
					throws InvocationTargetException {
				try {
					doFinish(containerName, fileName, monitor);					
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};

		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException
					.getMessage());
			return false;
		}
		return true;
	}
	
	
	////////////////////////////////
	// interface IWorkbenchWizard //
	////////////////////////////////
	
	/**
	 * We will accept the selection in the workbench to see if we can initialize
	 * from it.
	 * 
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
		if(!isClifAvaible()){
			MessageDialog.openError(getShell(),
					"CLIF Console Plug-in",
					"You need to install Birt to use Clif report template");
			dispose();
		}		
	}
	
	
	/////////////////////
	// private methods //
	/////////////////////
	
	/**
	 * The worker method. It will find the container, create the file if missing
	 * or just replace its contents, and open the editor on the newly created
	 * file.
	 * @param containerName
	 * @param fileName
	 * @param monitor
	 */
	private void doFinish(String containerName, String fileName,
			IProgressMonitor monitor) throws CoreException {
		
		/* Monitor needed by IFile while creation */
		monitor.beginTask("Creating " + fileName, 2);

		/* Test if container exist */
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource resource = root.findMember(new Path(containerName));
		if (resource == null || !resource.exists() || !(resource instanceof IContainer)) {
			throwCoreException("Container \"" + containerName
					+ "\" does not exist.");
		}
		
		/* Create file if it does not exist */
		IContainer container = (IContainer) resource;
		final IFile file = container.getFile(new Path(fileName));

		try {
			InputStream stream = openContentStream();
			
			if (!file.exists()) {
                file.create(stream, true, monitor);
            }
			stream.close();
		} catch (IOException e) {
			MessageDialog.openError(getShell(),
                    "CLIF Console Plug-in",
            		"Cannot create a new file");
		}
		
		// open the editor		
		monitor.worked(1);
		monitor.setTaskName(OPENING_FILE_FOR_EDITING);
//		getShell().getDisplay().asyncExec(new Runnable() {
		Display.getDefault().syncExec(new Runnable() {
			
			public void run() {
				IWorkbench workbench = PlatformUI.getWorkbench();
				IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
				IWorkbenchPage page = window.getActivePage();

				// switch perspective to BIRT
				IPerspectiveRegistry reg = workbench.getPerspectiveRegistry();
				page.setPerspective(reg.findPerspectiveWithId("org.eclipse.birt.report.designer.ui.ReportPerspective"));

				birtController.switchToBirtPerspective(page, file,settingPage);
			}
		});
		monitor.worked(1);
	}

	/**
	 * We will initialize file contents.
	 * @return file contents
	 */
	private InputStream openContentStream() {
		InputStream streamFromPage = null;
		URL url = Platform.find( Platform.getBundle("org.eclipse.birt.resources"),
				new Path( "/templates/blank_report.rpttemplate" ) );//$NON-NLS-1$
		if ( url != null )
		{
			try
			{
				streamFromPage = url.openStream( );
			}
			catch ( IOException e1 )
			{
				// ignore.
			}
		}
		return streamFromPage;
	}
	
	private void throwCoreException(String message) throws CoreException {
		IStatus status = new Status(IStatus.ERROR,
				"org.ow2.clif.analyze.lib.reportlink.CLIFReportPlugin",
				IStatus.OK, message, null);
		throw new CoreException(status);
	}


}
