/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.analyze.lib.oda;

import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Properties;

import org.eclipse.datatools.connectivity.oda.IConnection;
import org.eclipse.datatools.connectivity.oda.IParameterMetaData;
import org.eclipse.datatools.connectivity.oda.IQuery;
import org.eclipse.datatools.connectivity.oda.IResultSet;
import org.eclipse.datatools.connectivity.oda.IResultSetMetaData;
import org.eclipse.datatools.connectivity.oda.OdaException;
import org.eclipse.datatools.connectivity.oda.SortSpec;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;

import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.analyze.api.AnalyzerExternalAccess;
import org.ow2.clif.deploy.ClifAnalyzerAppFacade;
import org.ow2.clif.analyze.lib.oda.i18n.Messages;
import org.ow2.clif.analyze.lib.oda.util.querytextutil.QueryTextUtil;

/**
 * Flat file data provider's implementation of the ODA IQuery interface. It
 * supports single result set and no input parameters.
 */
public class ClifDataQuery implements IQuery
{

	/** The Constant DEFAULT_MAX_ROWS. */
	public static final int DEFAULT_MAX_ROWS = 1000;

	//the properties of the connection
	private Properties connProperties;

	// The max number of rows that can be read
	private int maxRows = 0;

	// The Connection instance associated with the Query.
	private IConnection connection = null;

	// The meta data of the query's result set.
	// It is available only after a query is prepared.
	private IResultSetMetaData resultSetMetaData = null;
	
	private ResultSetMetaDataHelper resultSetMetaDataHelper = null;
	
	private String testName;
	private String tableName;
	private String eventLabel;

    private AnalyzerExternalAccess analyzer = null;
	private ClifAnalyzerAppFacade clifAnalyzerApp;

	
	/**
	 * Constructor
	 * 
	 * @param connProperties
	 * @param host
	 * @param testName
	 * @throws OdaException 
	 */
	public ClifDataQuery(Properties connProperties, IConnection host, String testName ) throws OdaException
	{
		if ( connProperties == null	|| connProperties.getProperty( CommonConstants.CONN_HOME_DIR_PROP ) == null	|| host == null )
		{
			throw new OdaException( Messages.getString( "common_ARGUMENT_CANNOT_BE_NULL" ) );
		}
		this.connProperties = connProperties;
		this.connection=host;
		this.testName=testName;
		
	}
		
	
	//////////////////////
	// interface IQuery //
	//////////////////////

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#prepare(java.lang.String)
	 */
	public void prepare( String queryText ) throws OdaException
	{
		if ( queryText != null ) 
		{
			String query = QueryTextUtil.getQuery( queryText );
			String colInfo = QueryTextUtil.getColumnsInfo( queryText );
			validateOpenConnection( );
			String formattedQuery = formatQueryText( query );
			prepareMetaData( formattedQuery, colInfo );
		}
		else 
		{
			throw new OdaException( Messages.getString( "common_NULL_QUERY_TEXT" ) );
		}
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setAppContext(java.lang.Object)
	 */
	public void setAppContext( Object context ) throws OdaException
	{
		// do nothing; no support for pass-through application context
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setProperty(java.lang.String,
	 *      java.lang.String)
	 */
	public void setProperty( String name, String value ) throws OdaException
	{
		//	do nothing;
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#close()
	 */
	public void close( ) throws OdaException
	{
		this.maxRows = 0;
		this.connection = null;
		this.resultSetMetaData = null;
		this.tableName = "";
		this.testName = "";
		this.eventLabel = "";
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setMaxRows(int)
	 */
	public void setMaxRows( int max ) throws OdaException
	{
		this.maxRows = max;
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#getMaxRows()
	 */
	public int getMaxRows( ) throws OdaException
	{
		return this.maxRows;
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#getMetaData()
	 */
	public IResultSetMetaData getMetaData( ) throws OdaException
	{
		return this.resultSetMetaData;
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#executeQuery()
	 */
	public IResultSet executeQuery( ) throws OdaException
	{
		return new ResultSet(this.resultSetMetaData, this.testName,this.tableName,this.eventLabel );
	}
	
	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setInt(java.lang.String, int)
	 */
	public void setInt( String parameterName, int value ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setInt(int, int)
	 */
	public void setInt( int parameterId, int value ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setDouble(java.lang.String, double)
	 */
	public void setDouble( String parameterName, double value )
			throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setDouble(int, double)
	 */
	public void setDouble( int parameterId, double value ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setBigDecimal(java.lang.String, java.math.BigDecimal)
	 */
	public void setBigDecimal( String parameterName, BigDecimal value )
			throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setBigDecimal(int, java.math.BigDecimal)
	 */
	public void setBigDecimal( int parameterId, BigDecimal value )
			throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setString(java.lang.String, java.lang.String)
	 */
	public void setString( String parameterName, String value )
			throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setString(int, java.lang.String)
	 */
	public void setString( int parameterId, String value ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setDate(java.lang.String, java.sql.Date)
	 */
	public void setDate( String parameterName, Date value ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setDate(int,
	 *      java.sql.Date)
	 */
	public void setDate( int parameterId, Date value ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setTime(java.lang.String,
	 *      java.sql.Time)
	 */
	public void setTime( String parameterName, Time value ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setTime(int,
	 *      java.sql.Time)
	 */
	public void setTime( int parameterId, Time value ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setTimestamp(java.lang.String,
	 *      java.sql.Timestamp)
	 */
	public void setTimestamp( String parameterName, Timestamp value )
			throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setTimestamp(int,
	 *      java.sql.Timestamp)
	 */
	public void setTimestamp( int parameterId, Timestamp value )
			throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IQuery#setBoolean(java.lang.String, boolean)
     */
    public void setBoolean( String parameterName, boolean value )
            throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IQuery#setBoolean(int, boolean)
     */
    public void setBoolean( int parameterId, boolean value )
            throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IQuery#setNull(java.lang.String)
     */
    public void setNull( String parameterName ) throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /* (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.IQuery#setNull(int)
     */
    public void setNull( int parameterId ) throws OdaException
    {
        throw new UnsupportedOperationException();
    }

    /*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#findInParameter(java.lang.String)
	 */
	public int findInParameter( String parameterName ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#getParameterMetaData()
	 */
	public IParameterMetaData getParameterMetaData( ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#clearInParameters()
	 */
	public void clearInParameters( ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#setSortSpec(org.eclipse.datatools.connectivity.oda.SortSpec)
	 */
	public void setSortSpec( SortSpec sortBy ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IQuery#getSortSpec()
	 */
	public SortSpec getSortSpec( ) throws OdaException
	{
		throw new UnsupportedOperationException( );
	}
	
	
	/////////////////////
	// private methods //
	/////////////////////
		
	/**
	 * Validate whether the query's connection is open.
	 * 
	 * @throws OdaException
	 *             if connection is not open yet
	 */
	private void validateOpenConnection( ) throws OdaException
	{
		if ( connection.isOpen( ) == false )
		{
			throw new OdaException( Messages.getString( "common_CONNECTION_IS_NOT_OPEN" ) );
		}
	}

	/**
	 * Validate that there are no duplicate column names in a table
	 * 
	 * @param aCN
	 * @throws OdaException
	 */
	private void validateUniqueName( String[] aCN ) throws OdaException
	{
		for ( int i = 0; i < aCN.length; i++ )
		{
			if ( this.findOccuranceOfValueInStringArray( aCN[i], aCN ) > 1 )
			{
				throw new OdaException( Messages.getString( "query_SOURCE_DATA_ERROR" ) );
			}
		}
	}

	/**
	 * Return the number of occurance of a value in the given array
	 * 
	 * @param value
	 * @param array
	 * @return the number of occurancy of the given value int he given array
	 */
	private int findOccuranceOfValueInStringArray( String value, String[] array )
	{
		int count = 0;
		for ( int i = 0; i < array.length; i++ )
		{
			if ( value.trim( ).equalsIgnoreCase( array[i].trim( ) ) )
			{
				count++;
			}
		}
		return count;
	}

	/**
	 * Prepare the meta data which will be used in execution of a query text. It
	 * sets the value of two member variables: resultSetMetaData and
	 * currentTableName
	 * 
	 * @param queryText
	 * @throws OdaException
	 */
	private void prepareMetaData( String query, String savedSelectedColInfo )
			throws OdaException
	{
		if(query.trim().compareTo("") != 0){			
			String queryWithoutSelectKeyword = query.split("SELECT")[1];			
			String[] queryWithoutSelectAndFromKeyword = queryWithoutSelectKeyword.split("FROM");			
			String column = queryWithoutSelectAndFromKeyword[0];
			String table = queryWithoutSelectAndFromKeyword[1];
			
			validateSingleTableQuery(table);
			
			String[] splitColumn = column.split(",");			
			String[] splitTable = table.split("\\u002E");

			validateExistingTable(splitTable[0].trim());
			validateExistingEventLabel( splitTable[0].trim(), splitTable[1].trim());
			
			String[] col;
			String[] columnNames = new String[splitColumn.length];			
			String[] columnLabels = new String[splitColumn.length];

			for(int i = 0; i<splitColumn.length;i++){
				col=splitColumn[i].split("AS");
				columnNames[i]=col[0].trim();
				try{
					columnLabels[i]=col[1].trim();
				}catch(Exception e){
					columnLabels[i]="";
				}
			}

			if (isWildCard(columnNames))
			{
				validateWildCard(columnNames, columnLabels);
				clifAnalyzerApp = ClifAnalyzerAppFacade.getInstance();
		    	
		    	try{	
		    		Component analyzerComp = clifAnalyzerApp.getComponentByName("analyzer");
		    		analyzer = (AnalyzerExternalAccess) analyzerComp.getFcInterface(AnalyzerExternalAccess.ANALYZER_EXTERNAL_ACCESS);
		    		
		    	}
		    	catch (NoSuchInterfaceException ex)
		    	{
		    		System.err.println(ex);
		    	}
				columnNames = analyzer.getEventFieldLabels(this.testName, splitTable[0].trim(), splitTable[1].trim());
			}
			else
			{
				validateColumnNames(splitTable[0].trim(), splitTable[1].trim(), columnNames);
			}
			
			columnLabels = completeLabels(columnNames, columnLabels);
			
			this.tableName = splitTable[0].trim();
			this.eventLabel = splitTable[1].trim();
			this.resultSetMetaDataHelper = new ResultSetMetaDataHelper(splitTable[0].trim(),splitTable[1].trim(),columnNames,columnLabels);
			this.resultSetMetaData = new ResultSetMetaData( this.resultSetMetaDataHelper );
		}else{
			System.out.println("\"Empty Query\" in org.ow2.clif.analyze.lib.oda.ClifDataQuery.prepareMetaData() method");
		}
			
	}

	/**
	 * Replace inexisting columns labels by the name of the column.
	 * 
	 * @param tableName
	 * @param columnNames
	 * @param columnLabels
	 * @return the complete labels array
	 */
	private String[] completeLabels( String[] columnNames,String[] columnLabels)
			throws OdaException
	{
		String[] result = new String[columnNames.length];
		for(int i=0;i<columnNames.length;i++)
		{
			if(i >= columnLabels.length || columnLabels[i].equalsIgnoreCase(""))
			{
				result[i]=columnNames[i];
			}
			else
			{
				result[i]=columnLabels[i];
			}
		}
		return result;
	}
	
	/**
	 * Validate whether the given column names exist for the given table and event label.
	 * 
	 * @param tableName
	 * @param eventLabel
	 * @param columnNames
	 * @throws OdaException
	 *             if one column name doesn't exist
	 *             if one column name is not unique
	 */
	private void validateColumnNames( String tableName, String eventLabel, String[] columnNames)
			throws OdaException
	{
		clifAnalyzerApp = ClifAnalyzerAppFacade.getInstance();
    	
    	try{	
    		Component analyzerComp = clifAnalyzerApp.getComponentByName("analyzer");
    		analyzer = (AnalyzerExternalAccess) analyzerComp.getFcInterface(AnalyzerExternalAccess.ANALYZER_EXTERNAL_ACCESS);
    		
    	}
    	catch (NoSuchInterfaceException ex)
    	{
    		System.err.println(ex);
    	}
		String[] eventFieldsLabel = analyzer.getEventFieldLabels(this.testName, tableName, eventLabel);
		for(int i=0;i<eventFieldsLabel.length;i++)
		{
			if(findOccuranceOfValueInStringArray(eventFieldsLabel[i],columnNames)!=1)
			{
				throw new OdaException( "Query command is not valid" );
			}
			this.validateUniqueName(eventFieldsLabel);
		}
	}
	
	/**
	 * @param cCN
	 * @return true if the given string contains an asterisk
	 */
	private boolean isWildCard( String[] cCN )
	{
		for(int i=0;i<cCN.length;i++)
		{
			if ( cCN[i].equalsIgnoreCase( CommonConstants.KEYWORD_ASTERISK ) )
			{
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Validate whether the given column names and column labels
	 * 
	 * @param tableName
	 * @param eventLabel
	 * @throws OdaException
	 * 				if there is more than one column
	 * 				if the first column get an alias
	 */
	private void validateWildCard( String[] columnNames, String[] columnLabels)
			throws OdaException
	{
		if(columnNames.length>1)
		{
			throw new OdaException( "Query command is not valid" );
		}
		else if(!columnLabels[0].equalsIgnoreCase(""))
		{
			throw new OdaException( "Query command is not valid" );
		}
	}
	
	
	/**
	 * Validate whether the given event label is an existing event label for the given table name.
	 * 
	 * @param tableName
	 * @param eventLabel
	 * @throws OdaException
	 *             if the given event label doesn't exist
	 */
	private void validateExistingEventLabel( String tableName, String eventLabel )
			throws OdaException
	{
		clifAnalyzerApp = ClifAnalyzerAppFacade.getInstance();
    	
    	try{	
    		Component analyzerComp = clifAnalyzerApp.getComponentByName("analyzer");
    		analyzer = (AnalyzerExternalAccess) analyzerComp.getFcInterface(AnalyzerExternalAccess.ANALYZER_EXTERNAL_ACCESS);
    		
    	}
    	catch (NoSuchInterfaceException ex)
    	{
    		System.err.println(ex);
    	}
		String[] eventFieldsLabel = analyzer.getEventFieldLabels(this.testName, tableName, eventLabel);
		if (eventFieldsLabel==null || eventFieldsLabel.length==0)
		{
			throw new OdaException( "Query command is not valid" );
		}
	}
	
	
	/**
	 * Validate whether the given table name is an existing table name.
	 * 
	 * @param tableName
	 * @throws OdaException
	 *             if the given table name doesn't exist
	 */
	private void validateExistingTable( String tableName )
			throws OdaException
	{
		clifAnalyzerApp = ClifAnalyzerAppFacade.getInstance();
    	
    	try{	
    		Component analyzerComp = clifAnalyzerApp.getComponentByName("analyzer");
    		analyzer = (AnalyzerExternalAccess) analyzerComp.getFcInterface(AnalyzerExternalAccess.ANALYZER_EXTERNAL_ACCESS);
    		
    	}
    	catch (NoSuchInterfaceException ex)
    	{
    		System.err.println(ex);
    	}
		
		try{
			analyzer.getBladeProperties(this.testName, tableName);
		}catch(ClifException ce){
			throw new OdaException( "Query command is not valid" );
		}
	}
	
	/**
	 * Validate whether the given query segments contains a single table name.
	 * 
	 * @param table
	 * @throws OdaException
	 *             if the given query contains multiple table names
	 */
	private void validateSingleTableQuery( String table )
			throws OdaException
	{
		if ( table.split( CommonConstants.DELIMITER_COMMA_VALUE ).length != 1 )
		{
			throw new OdaException( Messages.getString( "query_DO_NOT_SUPPORT_CROSS_TABLE_QUERY" ) );
		}
	}
	
	/**
	 * Format the given query text. Eliminates redundant spaces and convert all
	 * keywords to uppercase.
	 * 
	 * @param queryText
	 * @return the formated query text from the given query text
	 */
	private String formatQueryText( String queryText )
	{
		String result = ""; //$NON-NLS-1$
		String[] temp = queryText.trim( ).split( CommonConstants.DELIMITER_SPACE );
		for ( int i = 0; i < temp.length; i++ )
		{
			if ( temp[i].equalsIgnoreCase( CommonConstants.KEYWORD_AS ) )
			{
				temp[i] = temp[i].toUpperCase( );
			}
			if ( temp[i].equalsIgnoreCase( CommonConstants.KEYWORD_FROM ) )
			{
				temp[i] = temp[i].toUpperCase( );
			}
			if ( temp[i].equalsIgnoreCase( CommonConstants.KEYWORD_SELECT ) )
			{
				temp[i] = temp[i].toUpperCase( );
			}
			result = result + temp[i] + CommonConstants.DELIMITER_SPACE;
		}
		return result.trim( );
	}
}
