/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.analyze.lib.oda;

import org.eclipse.datatools.connectivity.oda.IResultSetMetaData;
import org.eclipse.datatools.connectivity.oda.OdaException;
import org.ow2.clif.analyze.lib.oda.i18n.Messages;

/**
 * Flat file data provider's implementation of the ODA IResultSetMetaData
 * interface.
 */

public class ResultSetMetaData implements IResultSetMetaData
{
	private static final int STRING_TYPE_CODE = 12;
	private static final String STRING_TYPE_NAME = "String";

	private String[] columnNames = null;
	private String[] columnLabels = null;
	private String[] originalColumnNames = null;
	private String tableName;
	private String eventLabel;

	/**
	 * Constructor
	 * 
	 * @param rsmdHelper
	 * @throws OdaException 
	 */
	public ResultSetMetaData( ResultSetMetaDataHelper rsmdHelper ) throws OdaException
	{
		if( rsmdHelper == null )
			throw new OdaException( Messages.getString( "common_ARGUMENT_CANNOT_BE_NULL" ) );
		
		this.columnNames = rsmdHelper.getColumnNames( );
		this.originalColumnNames = rsmdHelper.getOriginalColumnNames( );
		this.columnLabels = rsmdHelper.getColumnLabels( );
		this.tableName = rsmdHelper.getTableName();
		this.eventLabel = rsmdHelper.getEventLabel();

		trimMetaDataStrings( );
	}
	
	
	//////////////////////////////////
	// interface IresultSetMetaData //
	//////////////////////////////////
	
	
	/*
	 * @see org.eclipse.datatools.connectivity.oda.IResultSetMetaData#getColumnCount()
	 */
	public int getColumnCount( ) throws OdaException
	{
		return this.columnNames.length;
	}


	/*
	 * @see org.eclipse.datatools.connectivity.oda.IResultSetMetaData#getColumnName(int)
	 */
	public String getColumnName( int index ) throws OdaException
	{
		validateColumnIndex( index );
		return this.columnNames[index - 1].trim( );
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IResultSetMetaData#getColumnLabel(int)
	 */
	public String getColumnLabel( int index ) throws OdaException
	{
		validateColumnIndex( index );
		// "null" in lower case represents null label value;
		if ( this.columnLabels == null
				|| this.columnLabels[index - 1].equals( "null" ) ) //$NON-NLS-1$
			return this.getColumnName( index ); // use column name instead

		return this.columnLabels[index - 1].trim( );
	}
	
	/*
	 * @see org.eclipse.datatools.connectivity.oda.IResultSetMetaData#getColumnType(int)
	 */
	public int getColumnType( int index )
	{
		return(STRING_TYPE_CODE);
	}
	
	/*
	 * @see org.eclipse.datatools.connectivity.oda.IResultSetMetaData#getColumnTypeName(int)
	 */
	public String getColumnTypeName( int index )
	{
		return(STRING_TYPE_NAME);
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IResultSetMetaData#getColumnDisplayLength(int)
	 */
	public int getColumnDisplayLength( int index ) throws OdaException
	{
		return(columnLabels[index].length());
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IResultSetMetaData#getPrecision(int)
	 */
	public int getPrecision( int index ) throws OdaException
	{
		return -1;
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IResultSetMetaData#getScale(int)
	 */
	public int getScale( int index ) throws OdaException
	{
		return -1;
	}

	/*
	 * @see org.eclipse.datatools.connectivity.oda.IResultSetMetaData#isNullable(int)
	 */
	public int isNullable( int index ) throws OdaException
	{
		return columnNullableUnknown;
	}

	
	/////////////////////
	// private methods //
	/////////////////////
		
	/**
	 *
	 */
	private void trimMetaDataStrings( )
	{
		assert columnNames.length == originalColumnNames.length
				&& originalColumnNames.length == columnLabels.length;

		for ( int i = 0; i < columnNames.length; i++ )
		{
			columnNames[i] = columnNames[i].trim( );
			columnLabels[i] = columnLabels[i].trim( );
			originalColumnNames[i] = originalColumnNames[i].trim( );
		}
		tableName.trim();
		eventLabel.trim();
	}	
	
	
	/**
	 * Evaluate whether the value of given column number is valid.
	 * 
	 * @param index
	 *            column number (1-based)
	 * @throws OdaException
	 *             if the given index value is invalid
	 */
	private void validateColumnIndex( int index ) throws OdaException
	{
		if ( index > getColumnCount( ) || index < 1 )
			throw new OdaException( Messages.getString( "resultSetMetaData_INVALID_COLUMN_INDEX" ) + index );
	}
}