package org.ow2.clif.deploy;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;

import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.ow2.clif.console.lib.egui.ClifConsolePlugin;
import org.ow2.clif.storage.api.AlarmEvent;
import org.ow2.clif.util.ExecutionContext;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.util.Fractal;


public class ClifAnalyzerAppFacade extends Observable implements Observer
{
	/** Fractal ADL definition file of the CLIF application to deploy */
	static final String CLIF_APPLICATION = "org.ow2.clif.console.lib.egui.ClifAnalyzerApp";


	protected Component clifAnalyzerApp;
	protected Component storage;
	protected Component analyzer;
	
	protected Map<String,Component> components = new HashMap<String,Component>();
	protected BindingController storageBc;
	protected BindingController analyzerBc;
	protected LifeCycleController clifAnalyzerAppLfc;
	protected ContentController clifAnalyzerAppCc;
	
	
	private static ClifAnalyzerAppFacade clifAnalyzerAppFacade = new ClifAnalyzerAppFacade("ClifRCPconsole", CLIF_APPLICATION);
	
	
	/**
	 * Create new ClifAnalyzerAppFacade with all components of a ClifApplication : 
	 * storage, analyzer...
	 * @param testName the name in the Registry to be associated with the
	 * resulting Clif Application.  
	 * @param appDefinition fully-qualified Fractal ADL definition file of the
	 * CLIF application to instantiate.
	 * @throws Error
	 */
	private ClifAnalyzerAppFacade(String testName, String appDefinition) throws Error
	{
		File clifPropsFile = new File(ResourcesPlugin.getPlugin().getStateLocation().append(ClifConsolePlugin.CLIF_PROPS).toOSString());
		if (!clifPropsFile.exists()){
			clifPropsFile = new File(ExecutionContext.getBaseDir() + File.separator + ExecutionContext.PROPS_PATH);
		}
		updateProperties(clifPropsFile);
		// Create Fractal Components
		try
		{
			clifAnalyzerApp = (Component)FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND
				).newComponent(
					appDefinition,
					null);
			setClifAnalyzerAppComponents();
			Fractal.getNameController(clifAnalyzerApp).setFcName(testName);
			
		}
		catch (Exception ex)
		{
			throw new Error("Could not create CLIF deployer: bad component configuration", ex);
		}
	}
	
	/**
     * set properties whose processing is specific to Eclipse console
     * @param ins InputStream which store CLIF properties
     */
    static private void setSpecialProperties(InputStream ins){
    	try
    	{
    		Properties props = new Properties();
    		props.load(ins);
    		String allParam = (String)props.get("clif.parameters");
    		String[] parameters = allParam.split(" ");
    		for (int i = 0; i < parameters.length; i++)
    		{
    			if(parameters[i].startsWith("-Dclif.codeserver.path"))
    			{
    				String[] property = parameters[i].split("=");
    				String prop = completeProperty(property[1]);
    				System.setProperty("clif.codeserver.path", prop);
    			}
    		}
    	}
    	catch (IOException e)
    	{
    		e.printStackTrace();
    	}
    }
    
    /**
     * Scans all paths separated with ";" and modifies relative paths to
     * absolute paths 
     * @param prop the property representing a list of paths
     */
    static private String completeProperty(String prop){
    	String completeProp="";
    	prop=prop.replaceAll("\"", "");
    	String[] property = prop.split(";");
    	for (int i=0;i<property.length;i++){
    		Path pathFile = new Path(property[i]);
    		if (pathFile.isAbsolute())
    		{
    			completeProp += property[i];
    		}
    		else
    		{
    			IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
    			pathFile = new Path(root.getLocation() + File.separator + property[i]);
    			completeProp += pathFile.toOSString();
    		}
    		if (i<property.length-1){
    			completeProp += ";";
    		}
    	}
    	return completeProp;
    }



	/**
	 * Get unique instance of ClifAnalyzerAppFacade
	 * @return clifAnalyzerAppFacade
	 */
	public static ClifAnalyzerAppFacade getInstance()
	{
		return clifAnalyzerAppFacade;
	}
	
	
	/**
	 * Set clifAnalyzerApp sub-components
	 * @throws ADLException
	 * @throws NoSuchInterfaceException
	 * @throws IllegalLifeCycleException
	 */
	private void setClifAnalyzerAppComponents()
		throws ADLException, NoSuchInterfaceException, IllegalLifeCycleException
	{
		Component[] subComp = Fractal.getContentController(clifAnalyzerApp).getFcSubComponents();
	
		for (int i=0 ; i<subComp.length ; ++i)
		{
			components.put(Fractal.getNameController(subComp[i]).getFcName(), subComp[i]);
			if (Fractal.getNameController(subComp[i]).getFcName().equals("storage"))
			{
				storage = subComp[i];
			}
			else if (Fractal.getNameController(subComp[i]).getFcName().equals("analyzer"))
			{
				analyzer = subComp[i];
			}
		}
		
		storageBc = Fractal.getBindingController(storage);
		analyzerBc = Fractal.getBindingController(analyzer);
		clifAnalyzerAppLfc = Fractal.getLifeCycleController(clifAnalyzerApp);
		clifAnalyzerAppCc = Fractal.getContentController(clifAnalyzerApp);
		clifAnalyzerAppLfc.startFc();
	}
	
	/**
	 * Get a component of the clifApplication by his Fractal name
	 * @param name the name of the component to find
	 * @return the reference of a component contained by the Clif Application whose name equals the
	 * String passed as parameter, or null if there is no such component.
	 */
	public Component getComponentByName(String name)
	{
		return components.get(name);
	}
	
	/**
	 * Get clifAnalyzerApp component
	 * @return Returns the clifAnalyzerApp component.
	 */
	public Component getClifAnalyzerApp() {
		return clifAnalyzerApp;
	}
	
	
    public void update(Observable supervisor, Object observation) {
	    if(countObservers() == 0 && observation instanceof AlarmEvent)
	    {
	        AlarmEvent alarm = (AlarmEvent)observation;
			if (alarm.argument != null)
			{
				if (alarm.argument instanceof Throwable)
				{
				    ((Throwable)alarm.argument).printStackTrace();
				    System.exit(1);
				}
			}
	    }
		setChanged();
		notifyObservers(observation);
	}


    static public void updateProperties(File clifProps)
    {
        try
        {
			ExecutionContext.setProperties(new FileInputStream(clifProps));
    		// set properties whose processing is specific to Eclipse console
    		setSpecialProperties(new FileInputStream(clifProps));
        }
        catch (Exception ex)
        {
        	System.err.println("Can't set system properties from file " + clifProps.getName());
        	ex.printStackTrace(System.err);
        }
    }
}
